# General

The code archive you received includes a Makefile to compile everything.
The command 'make help' tells you about the supported build targets. A
plain 'make' will compile everything.

Once compiled, you get three binaries:

- The Master Control Program (MCP) allows two computer programs to
  play against each other. The MCP orchestrates the game by sending
  the current game state to the players and receiving their moves.
  The MCP enforces these rules and makes sure the players behave.
- A keyboard player, which allows you to play the game yourself using
  keyboard input. Run 'make demo' to start up a game with two of these
  players.
- An empty file 'my_player.cc' where you should develop your game
  playing algorithm. Look at the keyboard player to learn how you receive
  the game state from the MCP and how to send your own move back. You
  may copy code from the keyboard player and all other source files, if
  you like. You can test your player as black or white against the
  keyboard player via 'make black' or 'make white', respectively.

  Please note that your player is NOT allowed to use the shared library
  game.so in any way, because it contains the game logic.


The official check whether you pass the assignment and also the
tournament is performed under a time and memory limit for your
player. You may use 1GB of memory and up to 60 seconds of think time
for one move (multiple threads are allowed, 'fork()' is not). These
limits are enforced by the MCP. You can test your player under these
conditions using 'make fight'.

If you implement a long-running algorithm in your player, you may
react to the SIGXCPU signal. The MCP sends this signal when the 60
second think timer has expired. If you react to this signal, you get
one extra second to report your move to the MCP before it SIGKILLs
you.

To test your player with a specific board, you can specify the board
state by using 'mcp -i <state>'.


# Haliotis

- Rules for Haliotis can be found online, at
    https://http://en.wikipedia.org/wiki/Abalone_(board_game)
    https://de.wikipedia.org/wiki/Abalone_(Spiel)
- In addition to the above rules Haliotis ends in a draw if the game runs
  longer than 100 turns, so your player has to make progress.
- Your task is to write your own player in my-player.cc, so that your
  player compiles, makes no invalid moves, and wins against a player
  that always chooses random moves. This random player is very weak at
  the game though.
- You can have a look at keyboard-player.cc, which is a player that allows
  you to play the game by entering moves on the keyboard. You can use
  this player to test yours, and you can take code from
  keyboard-player.cc. This is the only source you may copy code from.
  For the format to enter moves see below.
- You may usethe standard library for C/C++ and pthreads. If you
  want to use other libraries, please ask.
- You may not link against game.so.
- The master control program (mcp) starts the players and passes the
  board state to them.
- Type `make help` and `mcp -h` to get an overview about the commands.
- Do not hesitate to ask questions. If you have any questions about game
  rules or the programming task, please ask them on
    einfprakt@os.inf.tu-dresden.de

## Syntax for moves

The initial board, as written out by example-player, looks like this:

    I          W - W - W - W - W
              / \ / \ / \ / \ / \
    H        W - W - W - W - W - W
            / \ / \ / \ / \ / \ / \
    G      . - . - W - W - W - . - .
          / \ / \ / \ / \ / \ / \ / \
    F    . - . - . - . - . - . - . - .
        / \ / \ / \ / \ / \ / \ / \ / \
    E  . - . - . - . - . - . - . - . - .
        \ / \ / \ / \ / \ / \ / \ / \ /
    D    . - . - . - . - . - . - . - .   9
          \ / \ / \ / \ / \ / \ / \ /
    C      . - . - B - B - B - . - .   8
            \ / \ / \ / \ / \ / \ /
    B        B - B - B - B - B - B   7
              \ / \ / \ / \ / \ /
    A          B - B - B - B - B   6
    
                 1   2   3   4   5


The lines are denoted by letters, and the diagonals are denoted by
numbers. One possible 'inline' move would be to move B2 to C3, thus
moving the piece that was on C3 to D4. The notation for that is the
old and new position for the first moving piece in the row, separated
by a comma. In this example this would be "B2,C3". A possible next
move for white, moving just one piece, is H9,G8.

Moves that push pieces of the opponent have the same form: the
opponent's pieces are not explicitly stated in the move
representation. The board status is needed to determine how many
pieces have moved.

A possible 'broadside' move for black would be to move the frontmost
three pieces forward and to the left, thus moving C3 to D3, C4 to D4,
and C5 to D5. This is denoted by first giving the range of pieces that
should move, C3-C5, and then adding the target field for the first of
these. So the complete move is C3-C5,D3. C5-C3,D5 would be an
equivalent description.

keyboard-player expects moves to be entered in this exact form, with no
additional spaces. This is also the format that your player should use
to write its move to the mcp.
